import dns.resolver

from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.backends import default_backend

from base64 import b64decode, b64encode


def verify_sig(public_key, signature, data):
    """ Verifies a signature """
    try:
        pk = serialization.load_pem_public_key(
            public_key.encode(),
            backend=default_backend()
        )

        pk.verify(b64decode(signature), data.encode(), padding.PKCS1v15(), hashes.SHA256())

    except:
        return False

    return True

def generate_sig(private_key, data):
    """ Generates a signature on the passed in data """

    pk = serialization.load_pem_private_key(
        private_key.encode(),
        password=None,
        backend=default_backend()
        )

    sig = pk.sign(
        data.encode(),
        padding.PKCS1v15(),
        hashes.SHA256()
    )

    return b64encode(sig)


def get_publickey(domain):
        """ Gets a publickey from a zone """
        segments = {}

        pembits = ''

        records = dns.resolver.query(domain, 'TXT') # Get all text records
        record_strings = []
        for record in records:
            text = record.strings[0].decode('utf-8')
            record_strings.append(text)
            split_text = text.split(',')
            index = -1
            indexData = None
            for kv in split_text:
                if kv.startswith('p='):
                    index = int(kv[2:])
                elif kv.startswith('d='):
                    indexData = kv[2:]
                elif kv.startswith('a=') and kv != 'a=RS256':
                    return None
                elif kv.startswith('t=') and kv != 't=x509':
                    return None

            if index != -1 and indexData != None:
                segments[index] = indexData

        keys = segments.keys()
        keys = sorted(keys)

        # Concatenate all of the key segments
        for key in keys:
            pembits = pembits + segments[key].strip('\n').strip('\\n').strip()

        return '-----BEGIN PUBLIC KEY-----\n' + pembits + '\n-----END PUBLIC KEY-----\n'



from domainconnectzone.sigutil import verify_sig, generate_sig, get_publickey
from domainconnectzone.qsutil import qs2dict, qsfilter
from domainconnectzone.DomainConnect import DomainConnect, process_records, InvalidTemplate, HostRequired, InvalidSignature, MissingParameter, InvalidData, resolve_variables



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import io
import os

from distutils.cmd import Command
from setuptools import setup, find_packages

# ----------------------------------------
# Description
# ----------------------------------------
DESCRIPTION = 'Domain Connect Zone Utility'
LONG_DESCRIPTION = DESCRIPTION
LONG_DESCRIPTION_CONTENT_TYPE = ''
# Place contents of README into 'LONG_DESCRIPTION' for display on pypi.org project page.
here = os.path.abspath(os.path.dirname(__file__))
try:
        with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
                    LONG_DESCRIPTION = '\n' + f.read()
        LONG_DESCRIPTION_CONTENT_TYPE = 'text/plain',
except IOError:
        pass

# ----------------------------------------
# Clean
# ----------------------------------------
class CleanCommand(Command):
    description = "A better cleaner."
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        cmd_list = dict(
            build="rm -rf .eggs .pytest_cache dist *.egg-info",
            DS_Store="find ./ -name .DS_Store -delete;",
            pyc="find ./ -name '*.pyc' -delete;",
            empty_dirs="find ./ -type d -empty -delete;"
        )

        for key, cmd in cmd_list.items():
            os.system(cmd)


setup(
    name='domainconnectzone',
    version='3.3.0',
    description=DESCRIPTION,
    author='domainconnect.org',
    url='https://github.com/Domain-Connect/domainconnectzone',
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=[
        'cryptography>=1.8',
        'dnspython>=1.16',
        'IPy>=1.0'
    ],
    classifiers=[
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3"
    ]
)
